package org.apache.lucene.index;

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.IOException;

import org.apache.lucene.util.AttributeSource;
import org.apache.lucene.util.Bits;

/**
 * NOTE: this API is experimental and will likely change
 */

/** On obtaining a TermsEnum, you must first call next() */
public abstract class TermsEnum extends AttributeSource {

  /** Represents returned result from {@link TermsEnum.seek}.
   *  If status is SEEK_FOUND or SEEK_END, the text is
   *  unused; else (status is SEEK_NOT_FOUND), the text is the
   *  term that was actually found. */
  public final static class SeekResult {
    public static enum Status {END, FOUND, NOT_FOUND};
    public Status status;
    public TermRef term;
  }

  /** Seeks to the specified term.  Returns SeekResult to
   *  indicate whether exact term was found, a different
   *  term was found, or EOF was hit.  NOTE: the SeekResult
   *  instance may be reused across different calls to seek. */
  public abstract SeekResult seek(TermRef text) throws IOException;
  
  /** Increments the enumeration to the next element.
   *  Returns the resulting TermRef, or null if the end was
   *  hit.  The TermRef may be re-used between calls to
   *  next. */
  public abstract TermRef next() throws IOException;

  // nocommit: add term()?

  /** Returns the docFreq of the current Term in the enumeration.*/
  public abstract int docFreq();

  /** Get DocsEnum for the current term.  The returned
   *  DocsEnum may shared state with this TermsEnum, so you
   *  should not call this TermsEnum's {@link #seek} or
   *  {@link #next} until you are done using the
   *  DocsEnum. */
  public abstract DocsEnum docs(Bits skipDocs) throws IOException;
}

