package org.apache.lucene.search;

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.IOException;

import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.index.Terms;
import org.apache.lucene.index.TermRef;

/**
 * Subclass of FilteredTermEnum for enumerating all terms that match the
 * specified prefix filter term.
 * <p>
 * Term enumerations are always ordered by Term.compareTo().  Each term in
 * the enumeration is greater than all that precede it.
 *
 * @deprecated Use {@link PrefixTermsEnum} instead.
 */
public class PrefixTermsEnum extends FilteredTermsEnum {

  private final Term prefix;
  private final TermRef prefixRef;
  private boolean end = false;

  public PrefixTermsEnum(IndexReader reader, Term prefix) throws IOException {
    this.prefix = prefix;
    Terms terms = reader.fields().terms(prefix.field());
    prefixRef = new TermRef(prefix.text());
    if (terms != null) {
      init(terms.iterator(), prefixRef);
    } else {
      end = true;
    }
  }

  public String field() {
    return prefix.field();
  }

  public float difference() {
    return 1.0f;
  }

  protected boolean end() {
    return end;
  }
  
  protected Term getPrefixTerm() {
    return prefix;
  }

  protected boolean accept(TermRef term) {
    if (term.startsWith(prefixRef)) {
      return true;
    } else {
      end = true;
      return false;
    }
  }
}
