package org.apache.lucene.util.pfor;
/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* When using the Sun 1.6 jvm, the performance tests below (using doDecompPerfTest... methods)
 * should be run with the -server argument to the forked jvm that is used for the
 * junit tests by adding this line just before the 1st batchtest line
 * in common-build.xml:
      <jvmarg value="-server"/>
 * Using this -server may be slow for other tests, in particular for shorter tests.
 */ 
 
import junit.framework.TestCase;
import java.nio.IntBuffer;
import org.apache.lucene.util.LuceneTestCase;

/**
 * To be added: performance tests for decoding exceptions.
 */
public class TestPFor extends LuceneTestCase {
  private static boolean doDecompPerfTests = true;

  private static void doTestOffset(
    int[] input,
    int offset,
    int numFrameBits,
    int compressBufferSize,
    String testName) {
System.out.println();
System.out.println(testName);
    PFor pFor = new PFor();
    int actIntBufferSize = TestFrameOfRef.doNoBufferRun(input, offset, numFrameBits, pFor);
    assertEquals("IntBuffer size after noBuffer run compress()", compressBufferSize, actIntBufferSize);
    PFor pFor2 = new PFor();
    IntBuffer compressBuffer = TestFrameOfRef.compressToBuffer(input, offset, numFrameBits, actIntBufferSize, pFor2);
    // Decompress and verify against original input.
    PFor pFor3 = new PFor();
    TestFrameOfRef.deCompressFromBufferVerify(compressBuffer, input, offset, actIntBufferSize, pFor3);
  }

  private void doTest(int[] input, int numBits, int compressBufferSize) {
    int offset = 0;
    doTestOffset(input, offset, numBits, compressBufferSize, getName());
  }

  public void test02ExcByte1() {
    int[] input = {2}; // 1 byte exception
    int numBits = 1;
    int compressBufferSize = 2;
    doTest(input, numBits, compressBufferSize);
  }

  public void test02ExcByte2() {
    int[] input = {1,2}; // 1 byte exception
    int numBits = 1;
    int compressBufferSize = 2;
    doTest(input, numBits, compressBufferSize);
  }

  public void test02ExcByte3() {
    int[] input = {1,(1<<7)}; // 1 byte exception
    int numBits = 7;
    int compressBufferSize = 2;
    doTest(input, numBits, compressBufferSize);
  }

  public void test02ExcByte4() {
    int[] input = {1,(1<<7),0}; // 1 byte exception
    int numBits = 7;
    int compressBufferSize = 2;
    doTest(input, numBits, compressBufferSize);
  }

  public void test02ExcByte5() {
    int[] input = {1,(1<<7),0,65}; // 1 byte exception
    int numBits = 7;
    int compressBufferSize = 3;
    doTest(input, numBits, compressBufferSize);
  }

  public void test03ExcTwoByte1() {
    int[] input = {1<<8}; // 2 byte exception
    int numBits = 1;
    int compressBufferSize = 2;
    doTest(input, numBits, compressBufferSize);
  }

  public void test03ExcTwoByte2() {
    int[] input = {1<<8, 1}; // 2 byte exception
    int numBits = 1;
    int compressBufferSize = 2;
    doTest(input, numBits, compressBufferSize);
  }

  public void test03ExcTwoByte3() {
    int[] input = {1<<8, 1, 2}; // 2 byte exception
    int numBits = 3;
    int compressBufferSize = 2;
    doTest(input, numBits, compressBufferSize);
  }

  public void test03ExcTwoByte4() {
    int[] input = {1<<8, 1, 2}; // 2 byte exception
    int numBits = 6;
    int compressBufferSize = 3;
    doTest(input, numBits, compressBufferSize);
  }

  public void test03ExcTwoByte5() {
    int[] input = {1<<8, 1, 1<<9}; // two 2 byte exceptions
    int numBits = 2;
    int compressBufferSize = 3;
    doTest(input, numBits, compressBufferSize);
  }

  public void test03ExcTwoByte6() {
    int[] input = {1<<8, 1, 1<<9}; // two 2 byte exceptions
    int numBits = 6;
    int compressBufferSize = 3;
    doTest(input, numBits, compressBufferSize);
  }

  public void test05ExcThreeByte() {
    int[] input = {1<<16}; // 4 byte exception
    int numBits = 1;
    int compressBufferSize = 3;
    doTest(input, numBits, compressBufferSize);
  }

  public void test06ExcFourByte1() {
    int[] input = {1<<30}; // 4 byte exception, (1<<31 is negative, an assertion fails on negative values.
    int numBits = 1;
    int compressBufferSize = 3;
    doTest(input, numBits, compressBufferSize);
  }

  public void test06ExcFourByte2() {
    int[] input = {1<<30,0}; // 4 byte exception, (1<<31 is negative, an assertion fails on negative values.
    int numBits = 1;
    int compressBufferSize = 3;
    doTest(input, numBits, compressBufferSize);
  }

  public void test06ExcFourByte3() {
    int[] input = {1,1<<30,0}; // 4 byte exception, (1<<31 is negative, an assertion fails on negative values.
    int numBits = 6;
    int compressBufferSize = 3;
    doTest(input, numBits, compressBufferSize);
  }

  public void test08ForcedException1() {
    int[] input = {2,1,1}; // 2 exceptions, 1 byte
    int numBits = 1;
    int compressBufferSize = 2;
    doTest(input, numBits, compressBufferSize);
  }

  public void test09ForcedException2() {
    int[] input = {(1<<24),1,0}; // 2 exceptions, 4 byte
    int numBits = 1;
    int compressBufferSize = 4;
    doTest(input, numBits, compressBufferSize);
  }

  public void test10FirstException() {
    int[] input = {0,1,2,3,0,1,6,7,8}; // Test for not forcing first exception at index 4 (2nd value 0)
    int numBits = 2;
    int compressBufferSize = 3; //  3 exceptions from value 6
    doTest(input, numBits, compressBufferSize);
  }

  public void test11Series8Base3() { // This also tests for not forcing first exception
    int[] input = {0,1,2,3,4,5,6,7,8};
    int numBits = 3;
    int compressBufferSize = 3; // 1 exception
    doTest(input, numBits, compressBufferSize);
  }


  private void frameBitsForCompressionTest(int[] input, int expectedNumFrameBits) {
System.out.println();
System.out.println(getName());
    PFor pfor = new PFor();
    pfor.setUnCompressedData(input, 0, input.length);
    assertEquals("numFrameBits", expectedNumFrameBits, pfor.frameBitsForCompression());
  }

  public void test20frameBitsForCompression() {
    int[] input = {2};
    frameBitsForCompressionTest(input, 2);
  }

  public void test21frameBitsForCompression() {
    int[] input = {9,8,7,6,5,4,3,2,1,0};
    frameBitsForCompressionTest(input, 4);
  }

  public void test22frameBitsForCompression() {
    int[] input = {16000,16001,6,5,4,3,2,1,0};
    frameBitsForCompressionTest(input, 3);
  }

  private void doDecompPerfTest(int[] decomprData, int numBitsNoExceptions) {
    doDecompPerfTestNoExceptions(decomprData, numBitsNoExceptions);
    for (int nb = numBitsNoExceptions-1; nb >= numBitsNoExceptions-3; nb--) {
      if (nb <= 0)
        break;
      doDecompPerfTestExceptions(decomprData, nb);
    }
  }

  private void checkExceptions(int inputLength, int numBits, int compressBufferSize, boolean exceptionsPresent) {
    int sizeNoExceptions = 1 + (inputLength * numBits + 31) / 32;
    if (exceptionsPresent) {
      assertTrue(("Performance test with exceptions, IntBuffer  " + sizeNoExceptions + " < " + compressBufferSize),
                sizeNoExceptions < compressBufferSize);
      System.out.println("Nr of exception quadbytes: " + (compressBufferSize - sizeNoExceptions));
    } else {
      assertEquals("Performance test without exceptions, IntBuffer size", sizeNoExceptions, compressBufferSize);
    }
  }

  private void doDecompPerfTestNoExceptions(int[] decomprData, int numBits) {
    assert decomprData.length % 32 == 0 : decomprData.length;
    PFor pFor = new PFor();
    int comprDataSize = TestFrameOfRef.doNoBufferRun(decomprData, 0, numBits, pFor);
    checkExceptions(decomprData.length, numBits, comprDataSize, false);
    IntBuffer comprDataBuffer = TestFrameOfRef.compressToBuffer(decomprData, 0, numBits, comprDataSize, pFor);
    // Verify that decompression is correct:
    TestFrameOfRef.deCompressFromBufferVerify(comprDataBuffer, decomprData, 0, comprDataSize, pFor);

    if (! doDecompPerfTests) {
      return;
    }
System.out.println();
System.out.println(getName() + " starting, numFrameBits " + numBits);
    doDecompPerfTest(comprDataBuffer, decomprData.length);
  }

  private void doDecompPerfTestExceptions(int[] decomprData, int numBits) {
    assert decomprData.length % 32 == 0 : decomprData.length;
    PFor pFor = new PFor();
    int comprDataSize = TestFrameOfRef.doNoBufferRun(decomprData, 0, numBits, pFor);
    checkExceptions(decomprData.length, numBits, comprDataSize, true);
    IntBuffer comprDataBuffer = TestFrameOfRef.compressToBuffer(decomprData, 0, numBits, comprDataSize, pFor);
    // Verify that decompression is correct:
    TestFrameOfRef.deCompressFromBufferVerify(comprDataBuffer, decomprData, 0, comprDataSize, pFor);

    if (! doDecompPerfTests) {
      return;
    }
System.out.println();
System.out.println(getName() + " starting, numFrameBits " + numBits);
    doDecompPerfTest(comprDataBuffer, decomprData.length);
  }


  private void doDecompPerfTest(IntBuffer comprDataBuffer, int decomprSize) {
    TestFrameOfRef.doDecompPerfTestUsing(comprDataBuffer, decomprSize, new PFor(), getName());
  }

  public void tst92PerfPFor20_32Decompress() {
    for (int numBits = 20; numBits <= 32; numBits++) {
      int[] input = {
        1,(int)((1L<<numBits)-2),(int)((1L<<numBits)-1),127,262144,262143,4,7,
        16383,16382,35,37,63,2046,2047,60,
        9,4094,4095,14,511,131071,131072,9,
        23,21,8190,8191,226,255,0,17};
      doDecompPerfTestExceptions(input, numBits);
    }
  }

}
